class Admin::PagePartsController < OrbitBackendController
  
  layout "structure"

  open_for_admin

  def edit
    @id = params[:id]
    @part = PagePart.find(params[:id])
    @module_apps = ModuleApp.for_widget_select
    @module_app = @part.module_app || @module_apps.first
    @module_app_categories = @module_app.categories
    @module_app_tags = @module_app.tags
    @widget_paths = ModuleApp.find(@module_app.id).widgets.map{|name, data| [t(data["i18n"]), name]} rescue []
    if @part.widget_path.present?
      if @part.widget_path.eql?("default_widget")
        @checked_style = @part.widget_style
        @enabled_styles = @module_app.get_default_widget["enabled_styles"]
        @widget_fields = @module_app.widget_fields.collect do |widget_field| 
          label = I18n.t("#{@module_app.key}.default_widget.#{widget_field[0]}")
          [label, widget_field[0], class: widget_field[2]] 
        end
        @class_options = LIST[:widget_field_type].collect{|widget_field| [widget_field.humanize, widget_field]}
        @partial = 'default_widget'
      else
        @frontend_styles = @module_app.widgets[params[:val]]["style"] rescue nil
        @partial = 'custom_widget' if @frontend_styles.present?
      end
    end
    @data_count = @module_app.get_registration.get_data_count.to_a rescue []
    @no_orbit_bar = @side_bar = @no_header = true

    @r_tag = @part.public_r_tag.blank? ? LIST[:public_r_tags][0] : @part.public_r_tag
    @tag_objects = @r_tag.classify.constantize.all rescue nil
    if @r_tag.eql?('tag_cloud')
      @tag_objects = ModuleApp.where(has_tag: true)
    else
      @tag_objects = @r_tag.classify.constantize.all rescue nil
    end

    @html_class = "page-parts"
  end

  def get_page_setting_json
    begin
      part = PagePart.find(params[:id])
      m = {}
      m["main"] = part.module_app_id.to_s
      m["sub"] = part.widget_path
      m["category"] = part.category.present? ? [0, part.category.map{|c| c.to_s}] : [1]
      m["tags"] = part.tag.present? ? [0, part.tag.map{|c| c.to_s}] : [1]
      m["count"] = part.widget_data_count 
      render json: JSON.pretty_generate({module: m})
    rescue
      render :json => {error: ''}, status: 422
    end
  end

  def get_page_module_json
    modules = ModuleApp.for_widget_select.inject([]) do |module_apps, module_app|
      m = {}
      m["main"] = [module_app.label, module_app.id.to_s]
      m["sub"] = module_app.widgets.map{|name, data| [t(data["i18n"]), name]} rescue []
      m["category"] = module_app.categories.map{|category| [category.title, category.id.to_s] } rescue ''
      m["tags"] = module_app.tags.map{|tag| [tag.name, tag.id.to_s] } rescue ''
      m["count"] = module_app.get_registration.get_data_count.to_a
      module_apps << m
    end
    render json: JSON.pretty_generate({module: modules})
  end

  def get_display_style
    @part = PagePart.find(params[:id]) rescue nil
    @module_app = ModuleApp.find(params[:module_id]) rescue nil
    if @module_app
      if params[:val].eql?("default_widget")
        @checked_style = @part.widget_path.present? ? @part.widget_style : nil if @part && @part.widget_path.eql?("default_widget")
        @enabled_styles = @module_app.get_default_widget["enabled_styles"]
        @widget_fields = @module_app.widget_fields.collect do |widget_field| 
          label = I18n.t("#{@module_app.key}.default_widget.#{widget_field[0]}")
          [label, widget_field[0], class: widget_field[2]] 
        end
        @class_options = LIST[:widget_field_type].collect{|widget_field| [widget_field.humanize, widget_field]}
        @partial = 'default_widget'
      else
        @frontend_styles = @module_app.widgets[params[:val]]["style"] rescue nil
        @partial = 'custom_widget' if @frontend_styles.present?
      end
    end
  end

  def update
    @part = PagePart.find(params[:id])
    if @part.update_attributes(params[:page_part])
      set_children_sub_menu(@part) if @part.public_r_tag && @part.public_r_tag.eql?('sub_menu')
      flash.now[:notice] = t('update.success.content')
      respond_to do |format|
        format.html { 
          redirect_to admin_page_url(@part.page, edit: true)
        }
      end
    else
      @id = params[:id]
      @module_apps = ModuleApp.for_widget_select
      @module_app = @part.module_app || @module_apps.first
      @module_app_categories = @module_app.categories
      @module_app_tags = @module_app.tags
      @widget_paths = ModuleApp.find(@module_app.id).widgets.map{|name, data| [t(data["i18n"]), name]} rescue []
      if @part.widget_path.present?
        if @part.widget_path.eql?("default_widget")
          @checked_style = @part.widget_style
          @enabled_styles = @module_app.get_default_widget["enabled_styles"]
          @widget_fields = @module_app.widget_fields.collect do |widget_field| 
            label = I18n.t("#{@module_app.key}.default_widget.#{widget_field[0]}")
            [label, widget_field[0], class: widget_field[2]] 
          end
          @class_options = LIST[:widget_field_type].collect{|widget_field| [widget_field.humanize, widget_field]}
          @partial = 'default_widget'
        else
          @frontend_styles = @module_app.widgets[params[:val]]["style"] rescue nil
          @partial = 'custom_widget' if @frontend_styles.present?
        end
      end
      @data_count = @module_app.get_registration.get_data_count.to_a rescue []
      @no_orbit_bar = @side_bar = @no_header = true

      @r_tag = @part.public_r_tag.blank? ? LIST[:public_r_tags][0] : @part.public_r_tag
      @tag_objects = @r_tag.classify.constantize.all rescue nil
      if @r_tag.eql?('tag_cloud')
        @tag_objects = ModuleApp.where(has_tag: true)
      else
        @tag_objects = @r_tag.classify.constantize.all rescue nil
      end

      @html_class = "page-parts"

      render :action => "edit"
    end
  end

  def destroy
    @item = Page.find(params[:id])
    @item.destroy
    redirect_to admin_items_url( :parent_id => @item.parent_id )
  end
  
  def reload_r_tag_options
    @part = PagePart.find params[:id]
    @r_tag = params[:type]
    if params[:type].eql?('tag_cloud')
      @tag_objects = ModuleApp.where(has_tag: true)
    else
      @tag_objects = @r_tag.classify.constantize.all rescue nil
    end
    respond_to do |format|
      format.js  {}
    end
  end

  protected

  def set_children_sub_menu(part)
    part.page.children.each do |child|
      child_part = child.page_parts.detect{ |x| x.name.eql?(part.name) } rescue nil
      child_part.update_attributes(:kind => part.kind, :public_r_tag => part.public_r_tag, :public_r_tag_object_id => part.public_r_tag_object_id) rescue nil
    end
  end
  
end
