# encoding: utf-8

class Lab
  include Mongoid::Document
  include Mongoid::Timestamps
  include Mongoid::MultiParameterAttributes

  include OrbitModel::LanguageRestrict
  include OrbitModel::Status
  include OrbitTag::Taggable
  
  LANGUAGE_TYPES = [ "English", "Chinese" ]


  field :lab_title, localize: true
  field :location, localize: true
  field :participating_professor, localize: true
  field :participating_student, localize: true
  
  # has_and_belongs_to_many :tags, :class_name => "PersonalLabTag"
  
  field :year
  field :language
  field :keywords
  field :extension_no
  field :research_direction
  field :facility
  field :url
  field :note
  field :create_user_id, :type => BSON::ObjectId
  field :update_user_id, :type => BSON::ObjectId

  paginates_per 10
  
  has_many :lab_files, :autosave => true, :dependent => :destroy
  
  accepts_nested_attributes_for :lab_files, :allow_destroy => true
  
  # before_save :clean_checkboxs

  validates :lab_title, :at_least_one => true

  before_validation :add_http
  
  after_save :save_lab_files
  
  validates :url, :format => /^(http|https):\/\/(([a-z0-9]+([\-\.]{1}[a-z0-9]+)*\.[a-z]{2,5})|((25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\.){3}(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?))(:[0-9]{1,5})?(\/.*)?/i, :unless => Proc.new{self.url.blank?}

  def self.search( category_id = nil )
  
	if category_id.to_s.size > 0
	
	  find(:all, :conditions => {lab_category_id: category_id}).desc( :is_top, :title )

	else
	  
	  find(:all).desc( :is_top, :title)
	  
	end
	
  end


  def self.widget_datas
  
	where( :is_hidden => false ).desc(:is_top, :created_at)

  end
  
  def save_lab_files
	self.lab_files.each do |t|
	  if t.should_destroy
		t.destroy
	  end
    end
  end

  protected

  def add_http
    unless self.url.blank? || self.url[/^http:\/\//] || self.url[/^https:\/\//]
      self.url = 'http://' + self.url
    end
  end

  def clean_checkboxs
    self.tagged_ids.delete('')
  end
  
end